/*
 MickPlane.pde
 Processing 2D Plane Simulation
 
 Released under The MIT License
 
 Copyright (c) 2012 Mick West
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 
 */
 
 class MPlane
 {
   
   PVector pos;
   PVector vel;
   
   boolean alive = true;
   
   ParticleSystem ps;
   
   int next_particle;
   int particle_gap = 10;
   
   int lifetime;
   
   
   
   MPlane(PVector _pos, PVector _vel, int _lifetime)
   {
     pos = _pos;
     vel = _vel;
     ps = new ParticleSystem(100, new PVector(width/2,height/2,0));
     next_particle = 0;
     lifetime = _lifetime;
   }
   
   
    void display()
    {
      float x = pos.x/meters_per_pixel;
      float y = pos.y/meters_per_pixel;
      float w=10;
      float h=10;
    
      stroke(255);
      strokeWeight(2);
      line(x-w/2,y-h/2,x+w/2,y+h/2);
      line(x-w/2,y+h/2,x+w/2,y-h/2);
      
      next_particle -= 1;
      if (next_particle <= 0)
      {
        next_particle = particle_gap;
        ps.addParticle(x,y);
      }
      
      
      ps.update();
      ps.render();
   }
   
  void update()
  {
    pos.x = pos.x + vel.x * time_step; 
    pos.y = pos.y + vel.y * time_step;
    lifetime -= 1;
    if (lifetime < 0)
    {
      die();
    }
 
  } 
   
  boolean dead()
  {
    return !alive;
  }

  void die()
  {
    alive = false;
  }
   
 }
 
 class MPlaneManager
 {
   ArrayList planes;
   
   MPlaneManager()
   {
     planes = new ArrayList();
   }
   
   MPlane AddPlane(PVector A, PVector B, float speed, int lifetime)
   {
     PVector AB = PVector.sub(B,A);
     
     PVector start = PVector.sub(A,AB);
         
     PVector vel = AB;
     vel.normalize();
     
     vel.mult(speed);
     
     // start at A - AB, so we get the off-screen portion of the contrail
     MPlane p = new MPlane(start,vel,lifetime);
     planes.add(p);
     return p;  
   }
   
   void update()
   {
     
      for (int i = planes.size()-1; i >= 0; i--) 
      {
        MPlane p = (MPlane) planes.get(i);
          p.update();
      }
   prune();
     
   }
   
   void display()
   {
      for (int i = planes.size()-1; i >= 0; i--) 
      {
        MPlane p = (MPlane) planes.get(i);
          p.display();
      }
   
   }
   
   
  void prune()
  {
    for (int i = planes.size()-1; i >= 0; i--) 
    {
      MPlane p = (MPlane) planes.get(i);
      if (p.dead()) {
        planes.remove(i);
      }
    }
  }
   
 }
